import java.io.*;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.charset.Charset;

import com.mojang.nbt.*;
import net.minecraft.world.level.chunk.storage.RegionFile;

/**
 * Minecraft to OpenSim
 * - Linking Virtual Worlds
 * VRA 2012 Project
 * @author Michael Craft <mcraft@peak15.org>
 */
public class Mc2os {
	// Minecraft region file to read from
	public static final RegionFile REGION_FILE = new RegionFile(new File("/home/minecraft/minecraft_server/world/region/r.0.0.mca"));
	
	// Chunk we want from this region (chunk numbers are local to each region)
	public static final int[] CHUNK_POS = {0, 0};
	
	// Section we want from this chunk (sections are 16 high and count from bedrock at 0)
	public static final int SECTION_NUMBER = 4; // 4 corresponds to the 5th chunk up, starting with y:65
	
	// File to save output to
	public static final File output = new File("/home/michael/public_html/mc2os/output.txt");

	// Block IDs
	public static final byte AIR = 0;
	public static final byte STONE = 1;
	public static final byte GRASS = 2;
	public static final byte DIRT = 3;
	public static final byte COBBLESTONE = 4;
	public static final byte WOOD = 5;
	public static final byte SAND = 12;
	public static final byte GRAVEL = 13;
	
	// Main
	public static void main(String[] args) {
		// Open the file at the chunk we want
		DataInputStream is = REGION_FILE.getChunkDataInputStream(CHUNK_POS[0], CHUNK_POS[1]);
		
		// Read the chunk into memory.
		CompoundTag chunkData = null;
		try {
			chunkData = NbtIo.read(is);
			
			// Close the file.
			is.close();
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}


		/* This object is the chunk we are working with
		 * Chunks contain the following tags:
		 * Biomes
		 * Entities
		 * xPos
		 * LastUpdate
		 * zPos
		 * TerrainPopulated
		 * TileEntities
		 * HeightMap
		 * Sections*/
		CompoundTag chunk = chunkData.getCompound("Level");
		
		// Get a list of all vertical sections in this chunk.
		ListTag<? extends Tag> sections = chunk.getList("Sections");
		
		/* This object is a vertical slice of the chunk, a section.
		 * Sections contain the following tags:
		 * Data
		 * SkyLight
		 * BlockLight
		 * Y
		 * Blocks
		 */
		CompoundTag section = (CompoundTag) sections.get(SECTION_NUMBER);
		
		// And here's what we've been looking for, the actual block data! :D
		byte[] blocks = section.getByteArray("Blocks");
		
		// Now we'll iterate through to find block types that we want and then add them to a big string
		// Each line of the string will be of the form "x z y blockId"
		// y and z are flipped since opensim and minecraft don't agree on which is the vertical axis
		String outputString = "";
		for(int x=0; x < 10; x++) {
			for(int z=0; z < 10; z++) {
				for(int y=0; y < 8; y++) {
					// Get the block id at this address
					byte blockId = blocks[x + z*16 + y*256];
					// Add block types we want to the string
					if(blockId == AIR || blockId == STONE || blockId == GRASS || blockId == DIRT
							|| blockId == COBBLESTONE || blockId == WOOD
							|| blockId == SAND || blockId == GRAVEL) {
						outputString += Integer.toString(x);
						outputString += " " + Integer.toString(z);
						outputString += " " + Integer.toString(y);
						outputString += " " + Integer.toString(blockId) + "\n";
					}
				}
			}
		}
		
		// Now write the string out to a file
		try {
			FileOutputStream stream = new FileOutputStream(output);
			FileChannel fc = stream.getChannel();
			ByteBuffer bb = Charset.forName("UTF-8").encode(outputString);
			fc.write(bb);
			stream.close();
		}
		catch(IOException e) {
			e.printStackTrace();
		}
	}
}
