// ------------------------
// Constants and Globals
// ------------------------

// Block IDs
integer AIR = 0;
integer STONE = 1;
integer GRASS = 2;
integer DIRT = 3;
integer COBBLESTONE = 4;
integer WOOD = 5;
integer SAND = 12;
integer GRAVEL = 13;

// Buttons
integer pwrBtn;
integer resetBtn;

// Lookup table for blocks
list blockPos;
list blockNum;

// is an operation currently in place?
integer busy;

// http key
key http_request_id;

// ------------------------
// Utility Functions
// ------------------------

// gets a link number from a position
integer getBlockNum(vector pos) {
    integer index = llListFindList(blockPos, [pos]);
    return llList2Integer(blockNum, index);
}

// set the blockid by position
setBlockId(vector block, integer id) {
    integer bNum = getBlockNum(block);
    setBlockNumId(bNum, id);
}

// set the blockid by link number
setBlockNumId(integer num, integer id) {
    if(id == AIR) {
        llSetLinkPrimitiveParamsFast(num, [PRIM_TEXTURE, ALL_SIDES, "00000000-0000-2222-3333-100000001007", <1,1,1>, ZERO_VECTOR, 0]);
    }
    else if(id == STONE) {
        llSetLinkPrimitiveParamsFast(num, [PRIM_TEXTURE, ALL_SIDES, "00000000-0000-2222-3333-100000001012", <1,1,1>, ZERO_VECTOR, 0]);
    }
    else if(id == GRASS) {
        llSetLinkPrimitiveParamsFast(num, [PRIM_TEXTURE, ALL_SIDES, "00000000-0000-2222-3333-100000001013", <1,1,1>, ZERO_VECTOR, 0]);
    }
    else if(id == DIRT) {
        llSetLinkPrimitiveParamsFast(num, [PRIM_TEXTURE, ALL_SIDES, "b8d3965a-ad78-bf43-699b-bff8eca6c975", <1,1,1>, ZERO_VECTOR, 0]);
    }
    else if(id == COBBLESTONE) {
        llSetLinkPrimitiveParamsFast(num, [PRIM_TEXTURE, ALL_SIDES, "00000000-0000-1111-9999-000000000003", <1,1,1>, ZERO_VECTOR, 0]);
    }
    else if(id == WOOD) {
        llSetLinkPrimitiveParamsFast(num, [PRIM_TEXTURE, ALL_SIDES, "00000000-0000-2222-3333-100000001018", <1,1,1>, ZERO_VECTOR, 0]);
    }
    else if(id == SAND) {
        llSetLinkPrimitiveParamsFast(num, [PRIM_TEXTURE, ALL_SIDES, "sand", <1,1,1>, ZERO_VECTOR, 0]);
    }
    else if(id == GRAVEL) {
        llSetLinkPrimitiveParamsFast(num, [PRIM_TEXTURE, ALL_SIDES, "00000000-0000-2222-3333-100000001014", <1,1,1>, ZERO_VECTOR, 0]);
    }
}

// sets all blocks to air
clearBlocks() {
    integer len = llGetListLength(blockNum);
    integer n;
    for(n=0; n<len; n++) {
        setBlockNumId(llList2Integer(blockNum, n), AIR);
    }
}

// Set the busy state
setBusy() {
    busy = 1;
    llSetLinkColor(pwrBtn, <1.0, 1.0, 0.0>, ALL_SIDES);
}

// clear the busy state
clearBusyOn() {
    busy = 0;
    llSetLinkColor(pwrBtn, <0.0, 1.0, 0.0>, ALL_SIDES);
}

// clear the busy state to off
clearBusyOff() {
    busy = 0;
    llSetLinkColor(pwrBtn, <1.0, 0.0, 0.0>, ALL_SIDES);
}

// ------------------------
// Main
// ------------------------

default {
    state_entry() {
        llSay(0, "Initializing...");
        
        // Add the root prim
        blockPos += ZERO_VECTOR;
        blockNum += LINK_ROOT;
        
        llSetAlpha(1.0, ALL_SIDES);
        
        // Parse all linked prims
        integer num = llGetNumberOfPrims();
        integer n;
        for(n=2; n<=num; n++) {
            string name = llGetLinkName(n);
            
            if(name == "block") {
                // Find offset from the root block, this also happens to be the minecraft block position.
                vector pos = llList2Vector(llGetLinkPrimitiveParams(n, [PRIM_POSITION]), 0) - llGetPos();
                blockPos += pos;
                blockNum += n;
            }
            else if(name == "Power Button") {
                pwrBtn = n;
                llSetLinkColor(pwrBtn, <1.0, 1.0, 0.0>, ALL_SIDES);
            }
            else if(name == "Reset Button") {
                resetBtn = n;
            }
        }
        
        state off;
    }
}

state on {
    state_entry() {
        llSay(0, "On.");
        
        llSetLinkColor(pwrBtn, <0.0, 1.0, 0.0>, ALL_SIDES);
        
        llSetTimerEvent(20);
        if(!busy) {
            http_request_id = llHTTPRequest("http://home.peak15.org/~michael/mc2os/output.php", [], "");
        }
    }
    
    timer() {
        if(!busy) {
            http_request_id = llHTTPRequest("http://home.peak15.org/~michael/mc2os/output.php", [], "");
        }
    }
    
    http_response(key request_id, integer status, list metadata, string body) {
        if (request_id == http_request_id) {
            setBusy();
            llSay(0, "Refreshing...");
            
            list lines = llParseString2List(body, ["\n"], [""]);
            integer len = llGetListLength(lines);
            integer n;
            for(n=0; n<len; n++) {
                string line = llList2String(lines, n);
                list parts = llParseString2List(line, [" "], [""]);
                vector target = ZERO_VECTOR;
                target.x = (float) llList2String(parts, 0);
                target.y = (float) llList2String(parts, 1);
                target.z = (float) llList2String(parts, 2);
                setBlockId(target, (integer) llList2String(parts, 3));
            }
            
            clearBusyOn();
        }
    }
    
    touch_start(integer total) {
        integer link = llDetectedLinkNumber(0);
        
        // Power button, toggle off.
        if(link == pwrBtn) {
            state off;
        }
        // Reset button, clear blocks.
        else if(link == resetBtn && !busy) {
            setBusy();
            clearBlocks();
            clearBusyOn();
        }
    }
}

state off {
    state_entry() {
        llSay(0, "Off.");
        
        llSetLinkColor(pwrBtn, <1.0, 0.0, 0.0>, ALL_SIDES);
    }
    
    touch_start(integer total) {
        integer link = llDetectedLinkNumber(0);
        
        // Power button, toggle on.
        if(link == pwrBtn) {
            state on;
        }
        // Reset button, clear blocks.
        else if(link == resetBtn && !busy) {
            setBusy();
            clearBlocks();
            clearBusyOff();
        }
    }
}